@@ -1,4 +1,4 @@
-use poem::{handler, route, route::Route, Server};
+use poem::{handler, route, route::Route, RouteMethod, Server};
 
 #[handler]
 fn hello() -> String {
@@ -6,15 +6,12 @@ fn hello() -> String {
 }
 
 fn api() -> Route {
-    let mut route = route();
-    route.at("/hello").get(hello);
-    route
+    route().at("/hello", RouteMethod::new().get(hello))
 }
 
 #[tokio::main]
 async fn main() {
-    let mut app = route();
-    app.nest("/api", api());
+    let app = route().nest("/api", api());
     let server = Server::bind("127.0.0.1:3000").await.unwrap();
     server.run(app).await.unwrap();
 }
