@@ -1,7 +1,8 @@
 //! Route object and DSL
 
-use std::{collections::HashMap, sync::Arc};
+use std::sync::Arc;
 
+use fnv::FnvHashMap;
 use http::StatusCode;
 
 use crate::{
@@ -12,8 +13,7 @@ use crate::{
 /// Routing object
 #[derive(Default)]
 pub struct Route {
-    router: HashMap<Method, Router<Box<dyn Endpoint<Output = Response>>>>,
-    all_method_router: Router<Box<dyn Endpoint<Output = Response>>>,
+    router: Router<Box<dyn Endpoint<Output = Response>>>,
 }
 
 impl Route {
@@ -25,7 +25,7 @@ impl Route {
     /// # Example
     ///
     /// ```
-    /// use poem::{handler, route, web::Path};
+    /// use poem::{handler, route, web::Path, RouteMethod};
     ///
     /// #[handler]
     /// async fn a() {}
@@ -36,23 +36,24 @@ impl Route {
     /// #[handler]
     /// async fn c(Path(path): Path<String>) {}
     ///
-    /// let mut app = route();
-    ///
-    /// // full path
-    /// app.at("/a/b").get(a);
-    ///
-    /// // capture parameters
-    /// app.at("/b/:group/:name").get(b);
-    ///
-    /// // capture tail path
-    /// app.at("/c/*path").get(c);
+    /// let mut app = route()
+    ///     // full path
+    ///     .at("/a/b", RouteMethod::new().get(a))
+    ///     // capture parameters
+    ///     .at("/b/:group/:name", RouteMethod::new().get(b))
+    ///     // capture tail path
+    ///     .at("/c/*path", RouteMethod::new().get(c));
     /// ```
-    pub fn at<'a>(&'a mut self, path: &'a str) -> RouteMethod<'a> {
-        RouteMethod { path, router: self }
+    #[must_use]
+    pub fn at(mut self, path: &str, ep: impl IntoEndpoint) -> Self {
+        self.router
+            .add(path, Box::new(ep.into_endpoint().map_to_response()));
+        self
     }
 
     /// Nest a `Endpoint` to the specified path.
-    pub fn nest(&mut self, path: &str, ep: impl IntoEndpoint) {
+    #[must_use]
+    pub fn nest(mut self, path: &str, ep: impl IntoEndpoint) -> Self {
         let ep = Arc::new(ep.into_endpoint());
         let path = path.trim_end_matches('/');
 
@@ -97,11 +98,13 @@ impl Route {
             path.find('*').is_none(),
             "wildcards are not allowed in the nest path."
         );
-        self.all_method_router.add(
+        self.router.add(
             &format!("{}/*--poem-rest", path),
             Box::new(Nest(ep.clone())),
         );
-        self.all_method_router.add(path, Box::new(Root(ep)));
+        self.router.add(path, Box::new(Root(ep)));
+
+        self
     }
 }
 
@@ -109,7 +112,6 @@ impl Route {
 pub fn route() -> Route {
     Route {
         router: Default::default(),
-        all_method_router: Default::default(),
     }
 }
 
@@ -118,88 +120,89 @@ impl Endpoint for Route {
     type Output = Response;
 
     async fn call(&self, mut req: Request) -> Self::Output {
-        match self
-            .router
-            .get(req.method())
-            .and_then(|router| router.recognize(req.uri().path()).ok())
-            .or_else(|| self.all_method_router.recognize(req.uri().path()).ok())
-        {
-            Some(matches) => {
+        match self.router.recognize(req.uri().path()) {
+            Ok(matches) => {
                 req.state_mut().match_params.0.extend(matches.params.0);
                 matches.handler.call(req).await
             }
-            None => StatusCode::NOT_FOUND.into(),
+            Err(_) => StatusCode::NOT_FOUND.into(),
         }
     }
 }
 
-/// Used to set the endpoint of the HTTP methods.
-pub struct RouteMethod<'a> {
-    path: &'a str,
-    router: &'a mut Route,
+/// Routing object for HTTP methods
+#[derive(Default)]
+pub struct RouteMethod {
+    methods: FnvHashMap<Method, Box<dyn Endpoint<Output = Response>>>,
 }
 
-impl<'a> RouteMethod<'a> {
+impl RouteMethod {
+    /// Create a `RouteMethod` object.
+    pub fn new() -> Self {
+        Default::default()
+    }
+
     /// Sets the endpoint for specified `method`.
-    pub fn method(&mut self, method: Method, ep: impl IntoEndpoint) -> &mut Self {
-        self.router
-            .router
-            .entry(method)
-            .or_default()
-            .add(self.path, Box::new(ep.into_endpoint().map_to_response()));
+    pub fn method(mut self, method: Method, ep: impl IntoEndpoint) -> Self {
+        self.methods
+            .insert(method, Box::new(ep.into_endpoint().map_to_response()));
         self
     }
 
     /// Sets the endpoint for `GET`.
-    pub fn get(&mut self, ep: impl IntoEndpoint) -> &mut Self {
+    pub fn get(self, ep: impl IntoEndpoint) -> Self {
         self.method(Method::GET, ep)
     }
 
     /// Sets the endpoint for `POST`.
-    pub fn post(&mut self, ep: impl IntoEndpoint) -> &mut Self {
+    pub fn post(self, ep: impl IntoEndpoint) -> Self {
         self.method(Method::POST, ep)
     }
 
     /// Sets the endpoint for `PUT`.
-    pub fn put(&mut self, ep: impl IntoEndpoint) -> &mut Self {
+    pub fn put(self, ep: impl IntoEndpoint) -> Self {
         self.method(Method::PUT, ep)
     }
 
     /// Sets the endpoint for `DELETE`.
-    pub fn delete(&mut self, ep: impl IntoEndpoint) -> &mut Self {
+    pub fn delete(self, ep: impl IntoEndpoint) -> Self {
         self.method(Method::DELETE, ep)
     }
 
     /// Sets the endpoint for `HEAD`.
-    pub fn head(&mut self, ep: impl IntoEndpoint) -> &mut Self {
+    pub fn head(self, ep: impl IntoEndpoint) -> Self {
         self.method(Method::HEAD, ep)
     }
 
     /// Sets the endpoint for `OPTIONS`.
-    pub fn options(&mut self, ep: impl IntoEndpoint) -> &mut Self {
+    pub fn options(self, ep: impl IntoEndpoint) -> Self {
         self.method(Method::OPTIONS, ep)
     }
 
     /// Sets the endpoint for `CONNECT`.
-    pub fn connect(&mut self, ep: impl IntoEndpoint) -> &mut Self {
+    pub fn connect(self, ep: impl IntoEndpoint) -> Self {
         self.method(Method::CONNECT, ep)
     }
 
     /// Sets the endpoint for `PATCH`.
-    pub fn patch(&mut self, ep: impl IntoEndpoint) -> &mut Self {
+    pub fn patch(self, ep: impl IntoEndpoint) -> Self {
         self.method(Method::PATCH, ep)
     }
 
     /// Sets the endpoint for `TRACE`.
-    pub fn trace(&mut self, ep: impl IntoEndpoint) -> &mut Self {
+    pub fn trace(self, ep: impl IntoEndpoint) -> Self {
         self.method(Method::TRACE, ep)
     }
+}
 
-    /// Sets the endpoint for all methods.
-    pub fn all(&mut self, ep: impl IntoEndpoint) -> &mut Self {
-        self.router
-            .all_method_router
-            .add(self.path, Box::new(ep.into_endpoint().map_to_response()));
-        self
+#[async_trait::async_trait]
+impl Endpoint for RouteMethod {
+    type Output = Response;
+
+    async fn call(&self, req: Request) -> Self::Output {
+        match self.methods.get(req.method()) {
+            Some(ep) => ep.call(req).await,
+            None => StatusCode::NOT_FOUND.into(),
+        }
     }
 }
